@@ -217,6 +217,16 @@ class S3CopyToTable(rdbms.CopyToTable, _CredentialsMixin):
         """
         return ''
 
+    @property
+    def table_constraints(self):
+        """
+        Add extra table constraints, for example:
+
+        PRIMARY KEY (MY_FIELD, MY_FIELD_2)
+        UNIQUE KEY (MY_FIELD_3)
+        """
+        return ''
+
     @property
     def do_truncate_table(self):
         """
@@ -267,6 +277,16 @@ class S3CopyToTable(rdbms.CopyToTable, _CredentialsMixin):
         finally:
             cursor.close()
 
+    def create_schema(self, connection):
+        """
+        Will create the schema in the database
+        """
+        if '.' not in self.table:
+            return
+
+        query = 'CREATE SCHEMA IF NOT EXISTS {schema_name};'.format(schema_name=self.table.split('.')[0])
+        connection.cursor().execute(query)
+
     def create_table(self, connection):
         """
         Override to provide code for creating the target table.
@@ -290,12 +310,18 @@ class S3CopyToTable(rdbms.CopyToTable, _CredentialsMixin):
                     name=name,
                     type=type) for name, type in self.columns
             )
+
+            table_constraints = ''
+            if self.table_constraints != '':
+                table_constraints = ', ' + self.table_constraints
+
             query = ("CREATE {type} TABLE "
-                     "{table} ({coldefs}) "
+                     "{table} ({coldefs} {table_constraints}) "
                      "{table_attributes}").format(
                 type=self.table_type,
                 table=self.table,
                 coldefs=coldefs,
+                table_constraints=table_constraints,
                 table_attributes=self.table_attributes)
 
             connection.cursor().execute(query)
@@ -308,12 +334,18 @@ class S3CopyToTable(rdbms.CopyToTable, _CredentialsMixin):
                     type=type,
                     encoding=encoding) for name, type, encoding in self.columns
             )
+
+            table_constraints = ''
+            if self.table_constraints != '':
+                table_constraints = ',' + self.table_constraints
+
             query = ("CREATE {type} TABLE "
-                     "{table} ({coldefs}) "
+                     "{table} ({coldefs} {table_constraints}) "
                      "{table_attributes}").format(
                 type=self.table_type,
                 table=self.table,
                 coldefs=coldefs,
+                table_constraints=table_constraints,
                 table_attributes=self.table_attributes)
 
             connection.cursor().execute(query)
@@ -383,6 +415,27 @@ class S3CopyToTable(rdbms.CopyToTable, _CredentialsMixin):
             table=self.table,
             update_id=self.update_id)
 
+    def does_schema_exist(self, connection):
+        """
+        Determine whether the schema already exists.
+        """
+
+        if '.' in self.table:
+            query = ("select 1 as schema_exists "
+                     "from pg_namespace "
+                     "where nspname = lower(%s) limit 1")
+        else:
+            return True
+
+        cursor = connection.cursor()
+        try:
+            schema = self.table.split('.')[0]
+            cursor.execute(query, [schema])
+            result = cursor.fetchone()
+            return bool(result)
+        finally:
+            cursor.close()
+
     def does_table_exist(self, connection):
         """
         Determine whether the table already exists.
@@ -408,9 +461,12 @@ class S3CopyToTable(rdbms.CopyToTable, _CredentialsMixin):
         """
         Perform pre-copy sql - such as creating table, truncating, or removing data older than x.
         """
+        if not self.does_schema_exist(connection):
+            logger.info("Creating schema for %s", self.table)
+            self.create_schema(connection)
+
         if not self.does_table_exist(connection):
             logger.info("Creating table %s", self.table)
-            connection.reset()
             self.create_table(connection)
 
         if self.do_truncate_table:
